@@ -36,7 +36,7 @@ class Atom(object):
     flag_or_upper_re = re.compile(r'^(?P<hyphen>-{0,2})'
                                   r'($|[\da-zA-Z_][\da-zA-Z_\-]*$)')
     angular_bracket_re = re.compile(r'^<.*?>$')
-    options_re = re.compile('\[(?P<title>[^\s\]]*)options\]', re.IGNORECASE)
+    options_re = re.compile(r'\[(?P<title>[^\s\]]*)options\]', re.IGNORECASE)
 
     def __init__(self, *names, **kwargs):
         self.names = set(names)
@@ -158,6 +158,8 @@ class Option(Atom):
         # default = self.default
         names = self.names
         value = self.value
+        # if appeared_only and value is None:
+        #     return {}
         ref = self.ref
         logger.debug('%s in repeat: %s', self, in_repeat)
 
@@ -246,7 +248,7 @@ class Option(Atom):
 
         current = argv.current()
         if current is None:
-            logger.info('no argv left')
+            logger.debug('no argv left')
             return False
 
         # self_value = self.dump_value()
@@ -272,7 +274,9 @@ class Option(Atom):
 
         if self.ref is None:
             if attached_value is not None:
-                if argv.stdopt and argv.attachopt and not find_it.startswith('--'):
+                if (argv.stdopt
+                        and argv.attachopt
+                        and not find_it.startswith('--')):
                     logger.debug("%s put -%s pack to argv %s",
                                  self, attached_value, argv)
                     argv.insert(index, '-' + attached_value, _from)
@@ -298,7 +302,7 @@ class Option(Atom):
                 argv.current(index) == '--' and
                 argv.auto_dashes and
                 min(self.ref.arg_range()) > 0):
-            logger.info(
+            logger.debug(
                 '%s ref must fully match but failed because `--`', self)
             raise ExpectArgumentHitDoubleDashesExit(
                 ('/'.join(self.names) +
@@ -322,8 +326,8 @@ class Option(Atom):
         result = self.ref.match(to_match_ref_argv, repeat_match)
 
         if attached_value is not None and to_match_ref_argv:
-            logger.info('%s ref must fully match but failed for %s',
-                        self, to_match_ref_argv)
+            logger.debug('%s ref must fully match but failed for %s',
+                         self, to_match_ref_argv)
             raise ExpectArgumentExit(
                 '%s requires argument(s).' % ('/'.join(self.names)),
                 option=self.names,
@@ -348,12 +352,13 @@ class Option(Atom):
         return True
 
     def reset(self):
-        value = self.value
-        if value is not None:
-            if value is True or value is False:
-                self.value = False
-            else:
-                self.value = 0
+        # value = self.value
+        # if value is not None:
+        #     if value is True or value is False:
+        #         self.value = False
+        #     else:
+        #         self.value = 0
+        self.value = None
 
         if self.ref is not None:
             self.ref.reset()
@@ -497,6 +502,8 @@ class Command(Atom):
     def get_value(self, appeared_only, in_repeat):
         if in_repeat:
             value = int(self.value)
+            if appeared_only and value == 0:
+                return {}
         else:
             value = self.value
 
@@ -544,7 +551,7 @@ class Argument(Atom):
     def match(self, argv, repeat_match):
 
         if not repeat_match and (self.value is not None and self.value != []):
-            logger.info('%s already has a value %s', self, self.value)
+            logger.debug('%s already has a value %s', self, self.value)
             return True
 
         if argv.option_only:
@@ -619,8 +626,11 @@ class Argument(Atom):
         # Not work on py2.6
         # return {name: value for name in self.names}
         result = {}
-        for name in self.names:
-            result[name] = value
+        if appeared_only and self.value is None:
+            pass
+        else:
+            for name in self.names:
+                result[name] = value
         return result
 
     def get_sys_default_value(self, appeared_only, in_repeat):
@@ -927,9 +937,13 @@ class Unit(list):
                 if rest_args is None:
                     return False
                 if isinstance(element, Required):
-                    return self.balance_required_value(ellipsis_args, rest_args)
+                    return self.balance_required_value(
+                        ellipsis_args, rest_args
+                    )
                 else:
-                    return self.balance_optional_value(ellipsis_args, rest_args)
+                    return self.balance_optional_value(
+                        ellipsis_args, rest_args
+                    )
         return False
 
     def can_lend_value(self, element):
@@ -1120,15 +1134,85 @@ class Unit(list):
     def expand(self):
         cls = self.__class__
         repeat = self.repeat
+        logger.debug('expand %r', self)
         if len(self) == 1 and isinstance(self[0], Either):
-            return [cls(each, repeat=repeat) for each in self[0]]
+            coll = []
+            expanded = []
+            need_product = False
+            for each in self[0]:
+                # print(repr(each), isinstance(each, list) and
+                #       not isinstance(each, Unit))
+                # if isinstance(each, list) and not isinstance(each, Unit):
+                #     coll.append(each)
+                #     continue
+                fixed_each = cls(each, repeat=self.repeat).fix()
+                exp = fixed_each.expand()
+                if len(exp) > 1:
+                    need_product = True
+                coll.append(fixed_each)
+                expanded.append(exp)
+
+            result = []
+            for each in expanded:
+                result.extend(each)
+            return result
+            #     # if len(exp) == 1:
+            #     #     coll.append(exp[0])
+            #     # else:
+            #     #     coll.append(exp)
+            #
+            # logger.debug('either 1 %r -> %r', self, coll)
+            # if need_product:
+            #     producted = []
+            #     for each in product(*expanded):
+            #         producted.append(list(each))
+            # else:
+            #     producted = coll
+            # logger.debug('either 2 %r -> %r', coll, producted)
+            # result = [cls(*x, repeat=self.repeat).fix() for x in producted]
+            #
+            # logger.debug('either 3 %r -> %r', producted, result)
+            # logger.info('done expand %r -> %r', self, result)
+            # return result
+            # coll = []
+            # for each in self[0]:
+            #     # if isinstance(each, list) and not isinstance(each, Unit):
+            #     #     coll.append(each)
+            #     #     continue
+            #     fixed_each = cls(each).fix()
+            #     if fixed_each is not None:
+            #         coll.append(fixed_each.expand())
+            #
+            # logger.debug('either first expand %r -> %r', self, coll)
+            #
+            # result = []
+            # logger.debug('either product %r', coll)
+            # for each in product(*coll):
+            #     logger.warning(each)
+            #     # elem = cls(*each).fix()
+            #     # if elem is not None:
+            #     #     result.append(elem)
+            # logger.debug('expand either %r -> %r', self, result)
+            # return result
+
+            # temp = [cls(each, repeat=repeat).fix().expand()
+            #         for each in self[0]]
+            # logger.debug('temp=%r', temp)
+            # temp_coll = []
+            # for each in product(*temp):
+            #     logger.debug('product elem=%r', each)
+            #     temp_coll.append(list(each))
+            # result = temp_coll
+            # logger.debug('result=%r', result)
+            # logger.debug('expand %r -> %r', self, result)
+            # return result
 
         result = []
         for expanded in product(*(x.expand() for x in self)):
             # new = cls(*expanded, repeat=repeat)
             new = cls(*(e.copy() for e in expanded), repeat=repeat)
             result.append(new)
-
+        logger.debug('done expand %r -> %r', self, result)
         return result
 
     def matched(self):
@@ -1236,7 +1320,7 @@ class Optional(Unit):
     def match(self, argv, repeat_match):
         repeat = repeat_match or self.repeat
         logger.debug('matching %s with %s%s',
-                      self, argv, ', repeatedly' if repeat else '')
+                     self, argv, ', repeatedly' if repeat else '')
         func = (self.match_repeat if repeat else self.match_oneline)
 
         func(argv)
@@ -1288,7 +1372,7 @@ class OptionsShortcut(object):
         return self._hide.intersection(name)
 
     def arg_range(self):
-        logger.info('fixme: Unexpected call')
+        logger.debug('fixme: Unexpected call')
         return [0]
 
     def match(self, argv, repeat_match):
@@ -1300,8 +1384,8 @@ class OptionsShortcut(object):
                 lambda x: not hide.intersection(x[0].names), options):
 
             if not argv:
-                logger.info('argv run out before matching [options] %s(-%s)',
-                            options, self._hide)
+                logger.debug('argv run out before matching [options] %s(-%s)',
+                             options, self._hide)
                 return True
             logger.debug('[options] try %s matching %s', each, argv)
             each.match(argv, repeat_match)
@@ -1373,7 +1457,7 @@ class OptionsShortcut(object):
             'hide': tuple(obj._hide),
         }
 
-    formal_title_re = re.compile('[\-_]')
+    formal_title_re = re.compile('[\\-_]')
 
     @classmethod
     def convert_2_object(cls, dic, options, namedoptions):
@@ -1466,12 +1550,15 @@ class Either(list):
             result.update(each.arg_range())
         return list(result)
 
+    def __repr__(self):
+        return 'Either(%s)' % (', '.join(repr(x) for x in self))
+
 
 def convert_2_dict(obj):
     return obj.convert_2_dict(obj)
 
 
-formal_title_re = re.compile('[\-_]')
+formal_title_re = re.compile('[\\-_]')
 
 
 def convert_2_object(dic, options, namedoptions):
