@@ -52,7 +52,7 @@ class Parser(object):
         assert atom is not None
 
         if atom in ('-', '--'):
-            return self.parse_dash(atom , token)
+            return self.parse_dash(atom, token)
 
         if atom.startswith('-'):
             result = self.parse_option_with_arg(atom, token)
@@ -250,8 +250,12 @@ class Parser(object):
                         ref_token = Token()
                         if ref_current in '([':
                             ref_token.append(ref_current)
-                            ref_token.extend(token.till_end_bracket(ref_current))
-                            ref_token.append(')' if ref_current == '(' else ']')
+                            ref_token.extend(
+                                token.till_end_bracket(ref_current)
+                            )
+                            ref_token.append(
+                                ')' if ref_current == '(' else ']'
+                            )
                         else:
                             ref_token.extend(('(', ref_current, ')'))
 
@@ -262,12 +266,14 @@ class Parser(object):
 
                         if len(ref_ins) != 1:
                             raise DocpieError(
-                                '%s announced difference in Options(%s) and Usage(%s)' %
+                                ('%s announced difference in '
+                                 'Options(%s) and Usage(%s)') %
                                 (current, ins_in_opt, ref_ins))
 
                         if ins_in_opt.ref != ref_ins[0]:
                             raise DocpieError(
-                                '%s announced difference in Options(%s) and Usage(%s)' %
+                                ('%s announced difference in '
+                                 'Options(%s) and Usage(%s)') %
                                 (current, ins_in_opt, ref_ins))
 
                         ins = atom_class(*args, **{'ref': ins_in_opt.ref})
@@ -299,13 +305,13 @@ class Parser(object):
                     logger.debug('find %s for %s', exists_title, title)
                     return options
             else:
-                logger.info('%s options not found in %s',
-                            title, title_opt_2_ins)
+                logger.debug('%s options not found in %s',
+                             title, title_opt_2_ins)
                 raise DocpieError('%s options not found' % title)
 
         return sum(title_opt_2_ins.values(), [])
 
-    formal_title_re = re.compile('[\-_]')
+    formal_title_re = re.compile(r'[\-_]')
 
     def formal_title(self, title):
         return self.formal_title_re.sub(' ', title.lower()).strip()
@@ -384,7 +390,7 @@ class OptionParser(Parser):
         self.option_name = option_name
         self.option_split_re = re.compile(
             self.option_split_re_str.format(option_name),
-            flags= re.DOTALL if case_sensitive else (re.DOTALL | re.IGNORECASE)
+            flags=re.DOTALL if case_sensitive else (re.DOTALL | re.IGNORECASE)
         )
 
         self.raw_content = {}
@@ -553,12 +559,12 @@ class OptionParser(Parser):
                      '%(description_str)r' % locals())
         if description_str.strip():
             indent = len(opt_str.expandtabs()) + len(separater.expandtabs())
-            logger.info('indent: %s', indent)
+            logger.debug('indent: %s', indent)
         else:
             indent = 2 + len(cls.indent_re.match(
                                  opt_str.expandtabs()
                             ).groupdict()['indent'])
-            logger.info('indent: %s', indent)
+            logger.debug('indent: %s', indent)
         return {'option': opt_str.strip(), 'indent': indent}
 
     @classmethod
@@ -684,9 +690,9 @@ class OptionParser(Parser):
                                   opt_ins, this_range, current_range))
 
         if len(current_range) > 1:
-            logger.info('too many possibilities: '
-                        'option %s expect %s arguments',
-                        name, '/'.join(map(str, current_range)))
+            logger.debug('too many possibilities: '
+                         'option %s expect %s arguments',
+                         name, '/'.join(map(str, current_range)))
 
         # TODO: check if current_ins contain Command(not allowed in fact)
         opt_ins.ref = current_ins
@@ -703,7 +709,9 @@ class OptionParser(Parser):
 class UsageParser(Parser):
 
     angle_bracket_re = re.compile(r'(<.*?>)')
-    wrap_symbol_re = re.compile(r'(\[[^\]\s]*?options\]|\.\.\.|\||\[|\]|\(|\))')
+    wrap_symbol_re = re.compile(
+        r'(\[[^\]\s]*?options\]|\.\.\.|\||\[|\]|\(|\))'
+    )
     split_re = re.compile(r'(\[[^\]\s]*?options\]|\S*<.*?>\S*)|\s+')
     # will match '-', '--', and
     # flag ::= "-" [ "-" ] chars "=<" chars ">"
@@ -783,7 +791,7 @@ class UsageParser(Parser):
 
         reallen = len(dic['name'])
         replace = ''.ljust(reallen)
-        drop_name = match.expand('%s\g<sep>\g<section>' % replace)
+        drop_name = match.expand('%s\\g<sep>\\g<section>' % replace)
         self.formal_content = self.drop_started_empty_lines(drop_name).rstrip()
 
     def parse_2_instance(self, name):
@@ -907,7 +915,6 @@ class UsageParser(Parser):
             outside_opts, opt_shortcuts = \
                 self.find_optionshortcut_and_outside_option_names(ins)
 
-
             logger.debug(outside_opts)
 
             for opt_cut in opt_shortcuts:
