@@ -2,7 +2,8 @@ import sys
 import logging
 
 import warnings
-from docpie.error import DocpieExit
+import textwrap
+from docpie.error import DocpieExit, DocpieError
 from docpie.parser import UsageParser, OptionParser
 from docpie.element import convert_2_object, convert_2_dict
 from docpie.tokens import Argv
@@ -21,7 +22,7 @@ class Docpie(dict):
 
     # Docpie version
     # it's not a good idea but it can avoid loop importing
-    _version = '0.3.3'
+    _version = '0.4.3'
 
     option_name = 'Options:'
     usage_name = 'Usage:'
@@ -31,6 +32,7 @@ class Docpie(dict):
     auto2dashes = True
     name = None
     help = True
+    helpstyle = 'python'
     version = None
     stdopt = True
     attachopt = True
@@ -45,6 +47,7 @@ class Docpie(dict):
 
     def __init__(self, doc=None, help=True, version=None,
                  stdopt=True, attachopt=True, attachvalue=True,
+                 helpstyle='python',
                  auto2dashes=True, name=None, case_sensitive=False,
                  optionsfirst=False, appearedonly=False, namedoptions=False,
                  extra=None):
@@ -68,6 +71,7 @@ class Docpie(dict):
             namedoptions=namedoptions)
 
         self.help = help
+        self.helpstyle = helpstyle
         self.version = version
         self.extra = extra
 
@@ -84,9 +88,14 @@ class Docpie(dict):
             self.stdopt, self.attachopt, self.attachvalue, self.namedoptions)
 
         uparser.parse_content(self.doc)
-        self.usage_text = uparser.raw_content
+        self.usage_text = usage_text = uparser.raw_content
         # avoid usage contains "Options:" word
-        prefix, _, suffix = self.doc.partition(self.usage_text)
+        if usage_text is None:
+            assert self.usage_name.lower() not in self.doc.lower()
+            raise DocpieError(
+                'usage title %r not found in doc' % (self.usage_name,)
+            )
+        prefix, _, suffix = self.doc.partition(usage_text)
 
         oparser.parse(prefix + suffix)
         self.option_sections = oparser.raw_content
@@ -221,7 +230,8 @@ class Docpie(dict):
                         final_value = value_in_usage
                     if option.ref is None and each.repeat:
                         final_value = int(final_value or 0)
-                # just add this key-value. Note all option here never been matched
+                # just add this key-value.
+                # Note all option here never been matched
                 elif self.appeared_only:
                     continue
                 else:
@@ -242,7 +252,7 @@ class Docpie(dict):
                             arg_range = ref.arg_range()
                             # if min(arg_range) != 0:
                             #     # It requires at least a value
-                            #     logger.info('%s expects value', option)
+                            #     logger.debug('%s expects value', option)
                             #     raise DocpieExit(DocpieException.usage_str)
                             if max(arg_range) == 1:
                                 final_value = None
@@ -306,16 +316,17 @@ class Docpie(dict):
                         (argv_clone.auto_dashes and
                          list(argv_clone) == ['--'])):
                     argv_clone.check_dash()
-                    logger.info('matched usage %s / %s', each, argv_clone)
+                    logger.debug('matched usage %s / %s', each, argv_clone)
                     return each, argv_clone.dashes
 
-                logger.info('matching %s left %s, checking failed',
-                            each, argv_clone)
+                logger.debug('matching %s left %s, checking failed',
+                             each, argv_clone)
 
-            logger.info('failed matching usage %s / %s', each, argv_clone)
+            each.reset()
+            logger.debug('failed matching usage %s / %s', each, argv_clone)
 
         else:
-            logger.info('none matched')
+            logger.debug('none matched')
             raise DocpieExit(None)
 
     def check_flag_and_handler(self, token):
@@ -353,7 +364,9 @@ class Docpie(dict):
                             if not attachopt and index > 0:
                                 break
 
-                            logger.debug('check %s for %s', attached_name, auto)
+                            logger.debug(
+                                'check %s for %s', attached_name, auto
+                            )
 
                             stacked_name = '-' + attached_name
                             if stacked_name == auto:
@@ -370,25 +383,37 @@ class Docpie(dict):
                         found = (inputted == auto)
 
             if found:
-                logger.info('find %s, auto handle it', auto)
+                logger.debug('find %s, auto handle it', auto)
                 handler(self, auto)
 
     def exception_handler(self, error):
         logger.debug('handling %r', error)
 
         if self.option_sections:
-            help_msg = ('%s\n%s' %
-                        (self.usage_text,
+            help_msg = ('%s\n\n%s' %
+                        (self.usage_text.rstrip(),
                          '\n'.join(self.option_sections.values())))
         else:
             help_msg = self.usage_text
 
+        helpstyle = self.helpstyle
+        if helpstyle == 'python':
+            if self.option_sections:  # option section will help dedent
+                formated_help_msg = self.help_style_python(help_msg)
+            else:  # only need to dedent it
+                formated_help_msg = self.help_style_dedent(help_msg)
+        elif helpstyle == 'dedent':
+            formated_help_msg = self.help_style_dedent(help_msg)
+        else:
+            formated_help_msg = help_msg
+
         args = list(error.args)
         message = args[0]
         if message is not None:
-            help_msg = '%s\n\n%s' % (message, help_msg)
+            formated_help_msg = '%s\n\n%s' % (message, formated_help_msg)
 
-        args[0] = help_msg
+        # remove `\n` because `raise` will auto add
+        args[0] = formated_help_msg.rstrip()
         error = self.clone_exception(error, args)
         error.usage_text = self.usage_text
         error.option_sections = self.option_sections
@@ -432,19 +457,84 @@ class Docpie(dict):
     def help_handler(docpie, flag):
         """Default help(`--help`, `-h`) handler. print help string and exit.
 
-        By default, flag startswith `--` will print the full `doc`,
-        otherwith, print "Usage" section and "Option" section.
+        when help = 'short_brief', flag startswith `--` will print
+        the full `doc`, `-` for "Usage" section and "Option" section only.
+        when help = 'short_brief_notice', flag startswith `--` will print
+        the full `doc`, `-` for "Usage" section and "Option" section only,
+        with a message.
+        "Use `--help` to see the full help messsage" in the end
+
+        otherwith(default), print the full `doc`
         """
-        if flag.startswith('--'):
-            print(docpie.doc)
+        help_type = docpie.help
+        helpstyle = docpie.helpstyle
+        if helpstyle == 'python':
+            doc = Docpie.help_style_python(docpie.doc)
+        elif helpstyle == 'dedent':
+            doc = Docpie.help_style_dedent(docpie.doc)
+        # elif help_style == 'raw':
+        #     doc = Docpie.help_style_raw(docpie.doc)
         else:
-            print(docpie.usage_text)
-            option_sections = docpie.option_sections
-            if option_sections:
+            doc = docpie.doc
+
+        if help_type == 'short_brief':
+            if flag.startswith('--'):
+                print(doc)
+            else:
+                print(docpie.usage_text.rstrip())
+                option_sections = docpie.option_sections
+                if option_sections:
+                    print('')
+                    print('\n'.join(option_sections.values()))
+        elif help_type == 'short_brief_notice':
+            if flag.startswith('--'):
+                sys.stdout.write(doc)
+            else:
+                print(docpie.usage_text)
+                option_sections = docpie.option_sections
+                if option_sections:
+                    print('')
+                    print('\n'.join(option_sections.values()).rstrip())
                 print('')
-                print('\n'.join(option_sections.values()))
+                print('Use `--help` to see the full help messsage.')
+        else:
+            sys.stdout.write(doc)
         sys.exit()
 
+    @staticmethod
+    def help_style_python(docstring):
+        if not docstring:
+            return '\n'
+        # Convert tabs to spaces (following the normal Python rules)
+        # and split into a list of lines:
+        lines = docstring.expandtabs().splitlines()
+        # Determine minimum indentation (first line doesn't count):
+        # indent = sys.maxint
+        indent = None
+        for line in lines[1:]:
+            stripped = line.lstrip()
+            if stripped:
+                if indent is None:
+                    indent = len(line) - len(stripped)
+                else:
+                    indent = min(indent, len(line) - len(stripped))
+        # Remove indentation (first line is special):
+        trimmed = [lines[0].strip()]
+        # if indent < sys.maxint:
+        for line in lines[1:]:
+            trimmed.append(line[indent:].rstrip())
+        # Strip off trailing and leading blank lines:
+        while trimmed and not trimmed[-1]:
+            trimmed.pop()
+        while trimmed and not trimmed[0]:
+            trimmed.pop(0)
+        # Return a single string:
+        return '\n'.join(trimmed) + '\n'
+
+    @staticmethod
+    def help_style_dedent(docstring):
+        return textwrap.dedent(docstring)
+
     @staticmethod
     def version_handler(docpie, flag):
         """Default `-v` and `--version` handler. print the verison and exit."""
@@ -454,7 +544,7 @@ class Docpie(dict):
     # Because it's divided from dict
     # json.dump(docpie, default=docpie.convert_2_dict) won't work
     # so convert to dict before JSONlizing
-    def convert_to_dict(self):  # cls, self):
+    def to_dict(self):  # cls, self):
         """Convert Docpie into a JSONlizable dict.
 
         Use it in this way:
@@ -506,10 +596,10 @@ class Docpie(dict):
             'opt_names_required_max_args': self.opt_names_required_max_args
         }
 
-    convert_2_dict = convert_to_dict
+    convert_2_dict = convert_to_dict = to_dict
 
     @classmethod
-    def convert_to_docpie(cls, dic):
+    def from_dict(cls, dic):
         """Convert dict generated by `convert_2_dict` into Docpie instance
 
         You can do this:
@@ -560,7 +650,7 @@ class Docpie(dict):
 
         return self
 
-    convert_2_docpie = convert_to_docpie
+    convert_2_docpie = convert_to_docpie = from_dict
 
     def set_config(self, **config):
         """Shadow all the current config."""
@@ -617,8 +707,10 @@ class Docpie(dict):
                 ))
 
         if self.doc is not None and reinit:
-            logger.warning('You changed the config that requires re-initialized'
-                           ' `Docpie` object. Create a new one instead')
+            logger.warning(
+                'You changed the config that requires re-initialized'
+                ' `Docpie` object. Create a new one instead'
+            )
             self._init()
 
     def _formal_extra(self, extra):
@@ -638,20 +730,20 @@ class Docpie(dict):
                 alias.add(flag)
                 for each in alias:
                     if set_handler:
-                        logger.info('set %s hanlder %s', each, handler)
+                        logger.debug('set %s hanlder %s', each, handler)
                         self.extra[each] = handler
                     else:
-                        logger.info('remove %s hanlder', each)
+                        logger.debug('remove %s hanlder', each)
                         _hdlr = self.extra.pop(each, None)
                         logger.debug('%s handler %s removed', each, _hdlr)
                 break
         else:
             for flag in find_order:
                 if set_handler:
-                    logger.info('set %s hanlder', flag)
+                    logger.debug('set %s hanlder', flag)
                     self.extra[flag] = handler
                 else:
-                    logger.info('remove %s hanlder', flag)
+                    logger.debug('remove %s hanlder', flag)
                     self.extra.pop(flag, None)
 
     def find_flag_alias(self, flag):
