@@ -10,7 +10,8 @@ from docpie.error import DocpieExit, \
                          ExceptNoArgumentExit, \
                          ExpectArgumentExit, \
                          ExpectArgumentHitDoubleDashesExit, \
-                         AmbiguousPrefixExit
+                         AmbiguousPrefixExit, \
+                         DocpieError
 import json
 
 try:
@@ -241,6 +242,70 @@ class BasicTest(unittest.TestCase):
         true(args['-a'])
         true(args['-b'])
 
+    def test_no_match_no_option(self):
+        doc = """
+Usage:
+    trydocpie web [<port>]
+    trydocpie gen <folder>
+        """
+        expected = """\
+Usage:
+    trydocpie web [<port>]
+    trydocpie gen <folder>
+"""
+        argv = [__file__]
+        errout = None
+        try:
+            docpie(doc, argv)
+        except DocpieExit as e:
+            errout = str(e)
+        # let raise handle enter
+        self.assertEqual(errout + '\n', expected)
+
+    def test_no_match_no_option_dedent_correct(self):
+        doc = """
+    Usage:
+        trydocpie web [<port>]
+        trydocpie gen <folder>
+        """
+        expected = """\
+Usage:
+    trydocpie web [<port>]
+    trydocpie gen <folder>
+"""
+        argv = [__file__]
+        errout = None
+        try:
+            docpie(doc, argv)
+        except DocpieExit as e:
+            errout = str(e)
+        # let raise handle enter
+        self.assertEqual(errout + '\n', expected)
+
+    def test_no_match_no_option_with_description_dedent_correct(self):
+        doc = """
+whatever I said here.
+Doesn't matter.
+
+    Usage:
+        trydocpie web [<port>]
+        trydocpie gen <folder>
+
+Doesn't matter either.
+        """
+        expected = """\
+Usage:
+    trydocpie web [<port>]
+    trydocpie gen <folder>
+"""
+        argv = [__file__]
+        errout = None
+        try:
+            docpie(doc, argv)
+        except DocpieExit as e:
+            errout = str(e)
+        # let raise handle enter
+        self.assertEqual(errout + '\n', expected)
 
 class RunDefaultTest(unittest.TestCase):
 
@@ -1618,8 +1683,8 @@ Options: -a, --all=<here>
 
         doc = 'Usage: prog [--prefix --prefer --prepare] [<args>...]'
 
-        sys.argv = \
-            'prog --prepare --prefer --prefix -- --prefi --prefe --prep'.split()
+        sys.argv = ['prog', '--prepare', '--prefer', '--prefix',
+                    '--', '--prefi', '--prefe', '--prep']
         self.eq(doc, {'--prefix': True, '--prefer': True, '--prepare': True,
                       '--': True, '<args>': ['--prefi', '--prefe', '--prep']})
 
@@ -1656,7 +1721,7 @@ Options: -a, --all=<here>
         self.fail(doc)
 
         sys.argv = ['prog', '-vv', '--', '-v']
-        self.eq(doc, {'-v': 2, '<arg>':'-v' ,'--': True})
+        self.eq(doc, {'-v': 2, '<arg>': '-v', '--': True})
 
         doc = '''Usage: prog (<a> | <b> <c>) <d>'''
         doc2 = '''Usage: prog (<b> <c> | <a>) <d>'''
@@ -1693,10 +1758,12 @@ Options: -a, --all=<here>
             serialization preview
 
         Options:
-            -p, --path=<path>           save or load path or filename[default: ./]
+            -p, --path=<path>           save or load path or \
+filename[default: ./]
             -f, --format=<format>...    save format, "json" or "pickle"
                                         [default: json pickle]
-            -n, --name=<name>           save or dump filename without extension,
+            -n, --name=<name>           save or dump filename without
+                                        extension,
                                         default is the same as this file
             -h, -?                      print usage
             --help                      print this message
@@ -1733,12 +1800,15 @@ Options: -a, --all=<here>
 
         sys.argv = ['prog', '--repeat=1', '--repeat=2',
                     '--another-repeat=1', '--another-repeat=2']
-        self.eq(doc, {'--repeat': ['1', '2'], '--another-repeat': ['1', '2'],
+        self.eq(doc, {'--repeat': ['1', '2'],
+                      '--another-repeat': ['1', '2'],
                       'cmd': 0, '<arg>': [],
                       '--': False})
 
-        sys.argv = ['prog', '--repeat=1', '--repeat=2', '--repeat=3'
-                                                        '--another-repeat=1', '--another-repeat=2']
+        sys.argv = [
+            'prog', '--repeat=1', '--repeat=2', '--repeat=3'
+            '--another-repeat=1', '--another-repeat=2'
+        ]
         self.fail(doc)
 
         sys.argv = ['prog', '--repeat=1', '--repeat=2',
@@ -1794,7 +1864,7 @@ Options: -a, --all=<here>
             with StdoutRedirect() as f:
                 # help
                 with self.assertRaises(SystemExit) as e:
-                    docpie(doc)
+                    docpie(doc, helpstyle='raw')
 
             args = e.exception.args
             if len(args) == 1:
@@ -1802,7 +1872,7 @@ Options: -a, --all=<here>
             else:
                 self.assertEqual(args, ())
 
-            self.assertEqual(f.read(), 'Usage: prog [-abc]\n')
+            self.assertEqual(f.read(), 'Usage: prog [-abc]')
 
     def test_option_disorder_match(self):
         doc = 'Usage: prog -b -a'
@@ -1904,7 +1974,7 @@ Options:
         args = e.exception.args
         if len(args) == 1:
             if (platform.python_implementation().lower() == 'pypy' and
-                        sys.version_info == (2, 7, 3, 'final', 42)):
+                    sys.version_info == (2, 7, 3, 'final', 42)):
                 # it will give args == (0,), why?
                 return
 
@@ -2213,7 +2283,6 @@ OPTIONS:
         sys.argv = ['prog', '--long-opt']
         self.eq(doc, {'--': False, '--long': False, '--long-opt': True})
 
-
         doc = """
         Usage: prog --long=<opt>
                prog --long-opt
@@ -2232,7 +2301,7 @@ OPTIONS:
         try:
             docpie(doc)
         except BaseException as e:
-            self.assertEqual(str(e), doc)
+            self.assertEqual(str(e), "Usage: prog")
         else:
             raise RuntimeError('Should raise here')
 
@@ -2297,7 +2366,8 @@ class APITest(unittest.TestCase):
 
         Usage:
           options_shortcut_example.py --help | --version
-          options_shortcut_example.py [options] (do --after=<float> [--grace])... <port>...
+          options_shortcut_example.py [options] \
+(do --after=<float> [--grace])... <port>...
 
         Arguments:
           <port>                   give port
@@ -2317,7 +2387,14 @@ class APITest(unittest.TestCase):
           --after=<float>          do smth after <float> time
           --grace                  do smth with grace
         """
-        argv = ['options_shortcut_example.py', '-n', '6', 'do', '--after=4', 'a.txt']
+        argv = [
+            'options_shortcut_example.py',
+            '-n',
+            '6',
+            'do',
+            '--after=4',
+            'a.txt'
+        ]
         expect = {
             '--': False,
             '--after': ['4'],
@@ -2341,7 +2418,8 @@ class APITest(unittest.TestCase):
 
         Usage:
           options_shortcut_example.py --help | --version
-          options_shortcut_example.py [options] (do [--after=<float>] [--grace])... <port>...
+          options_shortcut_example.py [options] (do [--after=<float>] \
+[--grace])... <port>...
 
         Arguments:
           <port>                   give port
@@ -2425,7 +2503,6 @@ class APITest(unittest.TestCase):
         exception = cm.exception
         self.assertEqual(exception.args[0], 'exit')
 
-
     def test_extra_override_version(self):
         # with StderrRedirect() as f:
         sys.argv = ['prog', '-v']
@@ -2437,6 +2514,103 @@ class APITest(unittest.TestCase):
         exception = cm.exception
         self.assertEqual(exception.args[0], 'version~')
 
+    def test_new_help_default(self):
+        """https://github.com/TylerTemp/docpie/issues/10"""
+        doc = """
+        Header
+            Usage: prog [options]
+
+            Options: -a
+                     -b
+
+            Some description
+"""
+        argv = ['prog', '-h']
+        with StdoutRedirect() as f:
+            with self.assertRaises(SystemExit) as cm:
+                docpie(doc, help=True, argv=argv)
+
+        stdout = f.read()
+        self.assertIn('Some description', stdout)
+        self.assertIn('Header', stdout)
+        # self.assertEqual(doc, stdout)
+        default_doc_style = """\
+Header
+    Usage: prog [options]
+
+    Options: -a
+             -b
+
+    Some description
+"""
+        self.assertEqual(default_doc_style, stdout)
+        # assert False, repr(stdout)
+
+    def test_help_style_python(self):
+        doc = """
+        Header
+            Usage: prog [options]
+
+            Options: -a
+                     -b
+
+       Some description
+
+
+    """
+        argv = ['prog', '-h']
+        with StdoutRedirect() as f:
+            with self.assertRaises(SystemExit) as cm:
+                docpie(doc, help=True, helpstyle='python', argv=argv)
+
+        stdout = f.read()
+        self.assertIn('Some description', stdout)
+        self.assertIn('Header', stdout)
+        # self.assertEqual(doc, stdout)
+        python_style_doc = """\
+ Header
+     Usage: prog [options]
+
+     Options: -a
+              -b
+
+Some description
+"""
+        self.assertEqual(python_style_doc, stdout)
+        # assert False, repr(stdout)
+
+    def test_help_style_dedent(self):
+        doc = """
+        Header
+            Usage: prog [options]
+
+            Options: -a
+                     -b
+
+         Some description
+
+    """
+        argv = ['prog', '-h']
+        with StdoutRedirect() as f:
+            with self.assertRaises(SystemExit) as cm:
+                docpie(doc, help=True, helpstyle='dedent', argv=argv)
+
+        stdout = f.read()
+        self.assertIn('Some description', stdout)
+        self.assertIn('Header', stdout)
+        # self.assertEqual(doc, stdout)
+        python_style_doc = """
+Header
+    Usage: prog [options]
+
+    Options: -a
+             -b
+
+ Some description
+
+"""
+        self.assertEqual(python_style_doc, stdout)
+
 
 class NewErrorTest(unittest.TestCase):
 
@@ -2473,7 +2647,6 @@ class NewErrorTest(unittest.TestCase):
         error = self.with_raise(ExpectArgumentExit)
         self.assertIn('--prefix', error.option)
 
-
     def test_short_option_expect_args(self):
         sys.argv = ['prog', '-p']
         error = self.with_raise(ExpectArgumentExit)
@@ -2502,6 +2675,188 @@ class NewErrorTest(unittest.TestCase):
         self.assertEqual(set(('--prepare', '--prefix')), set(error.ambiguous))
 
 
+class IssueTest(unittest.TestCase):
+
+    def test_issue_5(self):
+        """https://github.com/TylerTemp/docpie/issues/5"""
+        sys.argv = ['prog', '--bb', 'B', '--cc', 'C']
+        doc = '''Usage: pie.py [--aa=AA|--bb=BB] --cc=CC'''
+        pie = docpie(doc)
+        self.assertEqual(
+            pie,
+            {'--': False, '--aa': None, '--bb': 'B', '--cc': 'C'}
+        )
+
+        sys.argv = ['prog', '--aa', 'A', '--cc', 'C']
+        doc = '''Usage: pie.py [--aa=AA|--bb=BB] --cc=CC'''
+        pie = docpie(doc)
+        self.assertEqual(
+            pie,
+            {'--': False, '--aa': 'A', '--bb': None, '--cc': 'C'}
+        )
+
+    def test_issue_6(self):
+        """https://github.com/TylerTemp/docpie/issues/6"""
+        doc = '''usage: pie.py [--aa=AA|[--bb=BB | --cc=CC]]'''
+        sys.argv = ['prog']
+        pie = docpie(doc)
+        self.assertEqual(
+            pie,
+            {'--': False, '--aa': None, '--bb': None, '--cc': None}
+        )
+
+        sys.argv = ['prog', '--aa', 'VA']
+        pie = docpie(doc)
+        self.assertEqual(
+            pie,
+            {'--': False, '--aa': 'VA', '--bb': None, '--cc': None}
+        )
+
+        sys.argv = ['prog', '--bb', 'VB']
+        pie = docpie(doc)
+        self.assertEqual(
+            pie,
+            {'--': False, '--aa': None, '--bb': 'VB', '--cc': None}
+        )
+
+        sys.argv = ['prog', '--cc=VC']
+        pie = docpie(doc)
+        self.assertEqual(
+            pie,
+            {'--': False, '--aa': None, '--bb': None, '--cc': 'VC'}
+        )
+
+        sys.argv = ['prog', '--aa=VA', '--bb', 'VB']
+        self.assertRaises(DocpieExit, docpie, doc)
+
+        sys.argv = ['prog', '--aa=VA', '--cc', 'VC']
+        self.assertRaises(DocpieExit, docpie, doc)
+
+        sys.argv = ['prog', '--bb=VB', '--cc', 'VC']
+        self.assertRaises(DocpieExit, docpie, doc)
+
+        doc = """Usage: prog [a | [b | [c | [d | e]]]]"""
+        sys.argv = ['prog']
+        pie = docpie(doc)
+        self.assertEqual(
+            pie,
+            {
+                '--': False,
+                'a': False,
+                'b': False,
+                'c': False,
+                'd': False,
+                'e': False
+            }
+        )
+
+        doc = """Usage: prog [a | [b | [c | [d | e]]]]"""
+        sys.argv = ['prog', 'd']
+        pie = docpie(doc)
+        self.assertEqual(
+            pie,
+            {
+                '--': False,
+                'a': False,
+                'b': False,
+                'c': False,
+                'd': True,
+                'e': False
+            }
+        )
+
+        doc = """Usage: prog [a | [b | [c | [d | e]]]]"""
+        sys.argv = ['prog', 'b', 'd']
+        self.assertRaises(DocpieExit, docpie, doc)
+
+    def test_issue_12(self):
+        """https://github.com/TylerTemp/docpie/issues/12"""
+        doc = """Usage: prog [a]..."""
+        sys.argv = ['prog']
+        pie = docpie(doc, appearedonly=True)
+        self.assertEqual(pie, {'--': False})
+
+        doc = """Usage: prog [-a]..."""
+        sys.argv = ['prog']
+        pie = docpie(doc, appearedonly=True)
+        self.assertEqual(pie, {'--': False})
+
+        doc = """Usage: prog [<a>]..."""
+        sys.argv = ['prog']
+        pie = docpie(doc, appearedonly=True)
+        self.assertEqual(pie, {'--': False})
+
+        doc = """Usage: prog [<a>]..."""
+        sys.argv = ['prog']
+        pie = docpie(doc, appearedonly=False)
+        self.assertNotEqual(pie, {'--': False})
+
+        doc = """Usage: prog [[-a]...]"""
+        sys.argv = ['prog']
+        pie = docpie(doc, appearedonly=True)
+        self.assertEqual(pie, {'--': False})
+
+        doc = """Usage:
+            create [--vbf_version=VBFVERSION]
+                   [--description=DESCRIPTION]...
+                   [--parameter_settings=(PARAMETER PARAMETERVALUE)]
+                   [([--vbt_sort | --vbt_last] --sign=ADDRESS)]
+        """
+        sys.argv = ['prog', '--sign=0x00', '--vbt_last']
+        pie = docpie(doc, appearedonly=True)
+        self.assertEqual(pie, {
+            '--': False,
+            '--sign': '0x00',
+            '--vbt_last': True})
+
+    def test_issue_11(self):
+        """https://github.com/TylerTemp/docpie/issues/11"""
+        doc = """
+`SCRIPT` DESCRIPTION
+
+Usage:
+ SCRIPT [options]
+
+Options:
+ -h, --help   show help
+"""
+
+        expect = """\
+Unknown option: --no-such.
+
+Usage:
+ SCRIPT [options]
+
+Options:
+ -h, --help   show help"""  # note: no linebreaker
+
+        with self.assertRaises(UnknownOptionExit) as cm:
+            docpie(doc, argv=['prog', '--no-such'])
+
+        exception = cm.exception
+        self.assertEqual(exception.args[0], expect)
+
+    def test_issue_9_error_handler(self):
+        """
+        https://github.com/TylerTemp/docpie/issues/9#issuecomment-354543053
+        """
+        doc = "Usage: prog whatever"
+
+        class Pie(Docpie):
+
+            usage_name = 'nosuch'
+
+        with self.assertRaises(DocpieError) as cm:
+            pie = Pie(doc, name='nosuch')
+            pie.docpie(argv=['prog', 'whatever'])
+
+        exception = cm.exception
+        exception_msg = exception.args[0]
+        self.assertIn('usage title', exception_msg)
+        self.assertIn('not found', exception_msg)
+        self.assertIn('nosuch', exception_msg)
+
+
 class Writer(StringIO):
 
     if sys.hexversion >= 0x03000000:
@@ -2553,6 +2908,7 @@ def case():
         unittest.TestLoader().loadTestsFromTestCase(RunDefaultTest),
         unittest.TestLoader().loadTestsFromTestCase(APITest),
         unittest.TestLoader().loadTestsFromTestCase(NewErrorTest),
+        unittest.TestLoader().loadTestsFromTestCase(IssueTest),
     )
 
 
@@ -2566,5 +2922,5 @@ def main():
 
 if __name__ == '__main__':
     # bashlog.stdoutlogger(None, bashlog.DEBUG, True)
-    unittest.main()
-    # main()
\ No newline at end of file
+    # unittest.main()
+    main()
